package tcc.haruki.domain.xtr;

import java.math.BigInteger;

import tcc.haruki.domain.XTRElem;
import tcc.haruki.exceptions.XTRInvalidParametersException;
import tcc.haruki.exceptions.XTRInvalidKeyException;

public class XTRPrivateKey {
	private XTRParameters params;
	private XTRElem trg;
	private BigInteger k;
	
	/**
	 * @param params parâmetros XTR
	 * @param trg Tr(g)
	 * @param k chave privada. 1 < k < q - 2
	 * @throws XTRInvalidParametersException 
	 * @throws XTRInvalidKeyException 
	 */
	public XTRPrivateKey(XTRParameters params, XTRElem trg, BigInteger k) throws XTRInvalidParametersException, XTRInvalidKeyException {
		this.params = params;
		this.trg = trg;
		this.k = k;
		
		if (this.params == null || this.trg == null || this.k == null) {
			throw new XTRInvalidKeyException("Algum valor da chave é nulo");
		}
		
		if (!this.params.isValid()) {
			throw new XTRInvalidParametersException();
		}
		
		if (this.k.compareTo(BigInteger.ZERO) < 0 || 
				this.k.compareTo(this.params.getQ().subtract(new BigInteger("2"))) > 0) {
			throw new XTRInvalidKeyException("k fora do intervalo permitido");
		}
	}
	
	/**
	 * p dos parâmetros XTR.
	 * @return p
	 */
	public BigInteger getP() {
		if (this.params == null) {
			return null;
		}
		return this.params.getP();
	}
	
	/**
	 * q dos parâmetros XTR.
	 * @return q
	 */
	public BigInteger getQ() {
		if (this.params == null) {
			return null;
		}
		return this.params.getQ();
	}
	
	/**
	 * 
	 * @return Tr(g)
	 */
	public XTRElem getTrg() {
		return trg;
	}
	
	/**
	 * A chave privada
	 * @return k
	 */
	public BigInteger getK() {
		return this.k;
	}
}
