package tcc.haruki.services;

import java.math.BigInteger;

import tcc.haruki.domain.XTRElem;
import tcc.haruki.domain.XTRTrincaElem;
import tcc.haruki.domain.xtr.XTRKey;
import tcc.haruki.exceptions.XTRInvalidParametersException;
import tcc.haruki.exceptions.XTRInvalidKeyException;
import tcc.haruki.exceptions.XTRNullParametersException;


public interface XTROperationService {
	/**
	 * Gera as chaves pública e privada para a utilização do XTR.
	 * @return uma XTRKey contendo a chave pública e a privada.
	 * @throws XTRNullParametersException
	 * @throws XTRInvalidParametersException
	 * @throws XTRInvalidKeyException
	 */
	public XTRKey generateKeys() throws XTRNullParametersException, XTRInvalidParametersException, XTRInvalidKeyException;
	
	/**
	 * Exponenciação simples de um elemento, tendo apenas seu traço.
	 * Dado Tr(g) e n, calcula Tr(g<sup>b</sup>)
	 * @param trg = Tr(g) = O elemento a ser exponenciado
	 * @param n O expoente
	 * @return Tr(g<sup>n</sup>)
	 */
	public XTRElem exponenciacaoSimples(XTRElem trg, BigInteger n);
	
	/**
	 * Recebe um elemento Tr(g) de GF(p<sup>2</sup>) e um elemento S<sub>k</sub> = 
	 * (Tr(g<sup>k-1</sup>), Tr(g<sup>k</sup>), Tr(g<sup>k+1</sup>)), inteiros a e b e
	 * calcula Tr(g<sup>a</sup> * g<sup>bk</sup>)
	 * 
	 * @param trg = Tr(g)
	 * @param sktrg = S<sub>k</sub>(Tr(g)) = (Tr(g<sup>k-1</sup>), Tr(g<sup>k</sup>), Tr(g<sup>k+1</sup>))
	 * @param a
	 * @param b
	 * @return Tr(g<sup>a</sup> * g<sup>bk</sup>)
	 */
	public XTRElem exponenciacaoDupla(XTRElem trg, XTRTrincaElem sktrg, BigInteger a, BigInteger b);
	
	
	/**
	 * Calcula uma trinca (c<sup>n-1</sup>, c<sup>n</sup>, c<sup>n+1</sup>) a
	 * partir do elemento c de GF(p<sup>2</sup>)
	 * 
	 * @param c
	 * @param n
	 * @return S<sub>n</sub>(c) = (c<sup>n-1</sup>, c<sup>n</sup>, c<sup>n+1</sup>)
	 */
	public XTRTrincaElem Sn(XTRElem c, BigInteger n);
	
	public BigInteger random1AtehQ();
}
