package tcc.haruki.services.impl;

import java.math.BigInteger;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.mockito.Mockito;

import tcc.haruki.domain.XTRElem;
import tcc.haruki.domain.GaloisField;
import tcc.haruki.domain.XTRTrincaElem;
import tcc.haruki.domain.xtr.XTRKey;
import tcc.haruki.domain.xtr.XTRParameters;
import tcc.haruki.exceptions.XTRInvalidParametersException;
import tcc.haruki.exceptions.XTRInvalidKeyException;
import tcc.haruki.exceptions.XTRNullParametersException;
import tcc.haruki.services.XTROperationService;

public class XTROperationServiceImplTest {
	
	private GaloisField gf;
	private XTROperationService service;
	private XTRParameters param;
	private XTRKey key;
	
	@Before
	public void setup() throws XTRNullParametersException, XTRInvalidParametersException, XTRInvalidKeyException {
		param = Mockito.mock(XTRParameters.class);
		Mockito.when(param.getP()).thenReturn(new BigInteger("175030811"));
		gf = new GaloisField(param.getP());
		Mockito.when(param.getQ()).thenReturn(new BigInteger("412807"));
		Mockito.when(param.getGF()).thenReturn(gf);
		Mockito.when(param.isValid()).thenReturn(true);
		service = new XTRDefaultOperationService(param);
		key = service.generateKeys();
	}

	@Test
	public void exponenciacaoSimplesTest() {
		XTRElem elem = new XTRElem(gf, new BigInteger("75"), new BigInteger("91"));
		XTRElem nElem = service.exponenciacaoSimples(elem, BigInteger.ONE);
		
		Assert.assertEquals(elem, nElem);
		
		XTRElem elem2 = new XTRElem(gf, new BigInteger("175025260"), new BigInteger("175022636"));
		XTRElem nElem2 = service.exponenciacaoSimples(elem, new BigInteger("2"));
		
		Assert.assertEquals(elem2, nElem2);
		
		XTRElem elem3 = new XTRElem(gf, new BigInteger("381419"), new BigInteger("709019"));
		XTRElem nElem3 = service.exponenciacaoSimples(elem, new BigInteger("3"));
		
		Assert.assertEquals(elem3, nElem3);
		
		XTRElem elem100 = new XTRElem(gf, new BigInteger("27163186"), new BigInteger("146561580"));
		XTRElem nElem100 = service.exponenciacaoSimples(elem, new BigInteger("100"));
		
		Assert.assertEquals(elem100, nElem100);
	}
	
	@Test
	public void consistenciaExponenciacaoTest() {
		XTRElem elem = new XTRElem(gf, new BigInteger("75"), new BigInteger("91"));
		XTRElem nElem2 = service.exponenciacaoSimples(elem, new BigInteger("2"));
		
		XTRElem nElem4 = service.exponenciacaoSimples(nElem2, new BigInteger("2"));
		
		XTRElem elem4 = service.exponenciacaoSimples(elem, new BigInteger("4"));
		Assert.assertEquals(elem4, nElem4);
		
	}
	
	@Test
	public void exponenciacaoDuplaTest() {
		XTRElem elem = new XTRElem(gf, new BigInteger("99444795"), new BigInteger("4150753"));
		BigInteger a = new BigInteger("2");
		BigInteger b = new BigInteger("5");
		BigInteger k = new BigInteger("7");
		BigInteger e = a.add(b.multiply(k));
		XTRElem resp = service.exponenciacaoSimples(elem, e);
		
		XTRTrincaElem sk = service.Sn(elem, k);
		XTRElem nElem = service.exponenciacaoDupla(elem, sk, a, b);
		
		Assert.assertEquals(resp, nElem);
	}
	
	@Test
	@Ignore
	public void listaGFElem() {
		XTRElem elem = new XTRElem(gf, new BigInteger("75"), new BigInteger("91"));
		BigInteger i = BigInteger.ZERO;
		
		System.out.println(param.getQ());
		
		for (int j = 0; j < 100; j++) {
			System.out.println(i + ": " + service.Sn(elem, i).get(0));
			i = i.add(BigInteger.ONE);
		}
	}
}
